#! /usr/bin/python

import os
import sys
import time
import PyTango
import signal
import traceback
import threading
import fcntl
from subprocess import Popen, PIPE
from PyQt4 import QtCore, QtGui
from DonkiScripting_ui import Ui_Dialog
from SyntaxHilighter import PythonHighlighter
import scriptingHelp
from scriptingHelp_rc import *
from pyEditor import PyEditWidget

ui = None

class checkScript(QtCore.QThread):
    add_log = QtCore.pyqtSignal(str,bool)
    def __init__(self, lwidget):
        QtCore.QThread.__init__(self)
        self.lwidget = lwidget
        self.go_on = True
        self.add_log.connect(self.lwidget.appendLogTxt)
        
    def run(self):
        while self.go_on:
            time.sleep(0.1)
            if (self.lwidget.p is None):
                continue
            # Check stdout
            while True:
                try:
                    out = self.lwidget.p.stdout.readline()
                    if len(out) == 0:
                        break
                    self.add_log.emit(out.strip(),False)
                except:
                    break
            # Check sterr
            while True:
                try:
                    out = self.lwidget.p.stderr.readline()
                    if len(out) == 0:
                        break
                    self.add_log.emit(out.strip(),False)
                except:
                    break
            # Check if script has finished
            if (self.lwidget.p.poll() is not None):
                self.lwidget.p = None
                self.add_log.emit("Script ended",True)
                self.lwidget.runButton.setEnabled(True)
                self.lwidget.abortButton.setEnabled(False)
                
        
    
################################################################################
class myDialog(Ui_Dialog):
    def __init__(self, script_path, parent):
        Ui_Dialog.__init__(self)
        self.script_path = script_path
        self.p = None
        self.default_header = "from donkipylib import *"
        self.parent = parent
        
    def setupWindowAndConnections(self):
        # Add logo
        self.logoLabel.setPixmap(QtGui.QPixmap(":images/DonkiPy.png"))

        # add Scripting Help
        self.HelpTab = QtGui.QWidget()
        self.HelpTab.setObjectName("HelpTab")
        self.gridLayout_help = QtGui.QGridLayout(self.HelpTab)
        self.gridLayout_help.setObjectName("gridLayout_help")
        my_path = os.path.dirname(os.path.realpath(__file__))
        self.HelpWidget = scriptingHelp.ScriptingHelpWidget(parent=None, lib_path=my_path+"/donkipylib")
        self.gridLayout_help.addWidget(self.HelpWidget, 0, 1, 1, 1)
        self.tabWidget.addTab(self.HelpTab, "Scripting Help")
        self.HelpWidget.icon.clear()
        # Add Python specific editor
        self.pyEditWidget = PyEditWidget(self.parent, self.script_path)
        self.scriptTextEdit = self.pyEditWidget.getTextEdit()
        self.scriptTextEdit.appendPlainText(self.default_header + "\n")
        self.gridLayout_3.addWidget(self.pyEditWidget, 1, 0, 1, 5)
        
        self.scriptTextEdit.setTabStopWidth(16)
        self.runButton.clicked.connect(self.runScript)
        self.abortButton.clicked.connect(self.abortScript)
        self.abortButton.setEnabled(False)
        self.tabWidget.setCurrentIndex(0)
        self.highlight = PythonHighlighter(self.scriptTextEdit.document())
        self.check_thread = checkScript(self)
        self.check_thread.start()


        
    def tabChanged(self,index):
        pass

    def runScript(self):
        #self.HelpWidget.show()
        try:
            if self.p is not None and self.p.poll() == None:
                self.errorMessage("Script already executing")
                return
            if not str(self.pyEditWidget.fileName).endswith(".py"):
                self.errorMessage("No active script found")
                return
            if self.default_header not in self.scriptTextEdit.toPlainText():
                new_text = "%s\n%s" % (self.default_header,self.scriptTextEdit.toPlainText())
                self.scriptTextEdit.setPlainText(new_text)
            self.pyEditWidget._saveScript(forced=True)
            scriptlines =  self.scriptTextEdit.toPlainText().split('\n')
            if map(str,scriptlines) == ['']:
                return
            my_path = os.path.dirname(os.path.realpath(__file__))
            donki_pypath = "%s/donkipylib" % my_path 
            inp_list = ["python", '-u', self.pyEditWidget.fileName]
            if 'PYTHONPATH' not in os.environ.keys():
                new_PYTHONPATH = donki_pypath
            else:
                new_PYTHONPATH = "%s:%s" % (os.environ['PYTHONPATH'],donki_pypath)
            env_dict = {'PYTHONPATH':new_PYTHONPATH, 'DONKIPY_LIBPATH':donki_pypath}
            self.p = Popen(inp_list,cwd=my_path,stdout=PIPE,stderr=PIPE,bufsize=0,env=env_dict)
            fd = self.p.stdout.fileno()
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)
            fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
            fd = self.p.stderr.fileno()
            fl = fcntl.fcntl(fd, fcntl.F_GETFL)
            fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
            self.logpanel.clear()
            self.appendLogTxt("Script Started",True)
            self.tabWidget.setCurrentIndex(1)
            self.runButton.setEnabled(False)
            self.abortButton.setEnabled(True)
        except:
            self.errorMessage(traceback.format_exc())

    def abortScript(self):
        if self.p is None:
            return
        if hasattr(self.p, 'send_signal'):
            #Python > 2.5
            self.p.send_signal(signal.SIGABRT)
        else:
            #Python 2.5
            os.kill(self.p.pid, signal.SIGABRT) 

    def appendLogTxt(self,txt , timedate = False):
        if timedate:
            self.logpanel.appendPlainText("%s %s" % (time.asctime(),txt))
        else:
            self.logpanel.appendPlainText(txt)
            

    def close_and_exit(self):
        self.check_thread.go_on = False
        if self.p is not None and self.p.poll() == None:
            self.abortScript()

    def errorMessage(self, err_message):
            msg = QtGui.QMessageBox()
            msg.setIcon(QtGui.QMessageBox.Warning)
            msg.setText(err_message)
            msg.setWindowTitle("Error")
            msg.setStandardButtons(QtGui.QMessageBox.Ok)
            retval = msg.exec_()

		

def sigint_handler( *args):
    """Handler for the SIGINT signal."""
    global ui
    ui.close_and_exit()
    QtGui.QApplication.quit()


def main():
    app = QtGui.QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon(":images/DonkiPy.png"))
    Dialog = QtGui.QDialog()
    if len(sys.argv) < 2:
        user_script_path = "."
    else:
        user_script_path = sys.argv[1]
    ui = myDialog(user_script_path, Dialog)
    ui.setupUi(Dialog)
    ui.setupWindowAndConnections()
    signal.signal(signal.SIGINT, sigint_handler)
    signal.signal(signal.SIGTERM, sigint_handler)
    # Next line enables CTRL-C abort
    #signal.signal(signal.SIGINT, signal.SIG_DFL)
    Dialog.show()
    try:
        app.exec_()
    finally:
        ui.close_and_exit()


if __name__ == "__main__":
    main()
