# -*- coding: utf-8 -*-

from __future__ import with_statement

try:
    # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
    # -> pythoncom must be imported first, otherwise py2exe's boot_com_servers
    #    will raise an exception ("ImportError: DLL load failed [...]") when
    #    calling any of the QFileDialog static methods (getOpenFileName, ...)
    import pythoncom #@UnusedImport
except ImportError:
    pass

from PyQt4.QtGui import (QDialog, QVBoxLayout, QLabel, QHBoxLayout,
                                QFileSystemModel, QMessageBox, QInputDialog,
                                QLineEdit, QMenu, QWidget, QToolButton,
                                QFileDialog, QToolBar, QTreeWidget,QGridLayout,
                                QTreeWidgetItem, QDrag, QTextBrowser,QPixmap,
                                QIcon, QFont, QColor)
from PyQt4.QtCore import (Qt, SIGNAL, QMimeData, QSize, QDir,QUrl)

import imp
import os, sys, re
import os.path as osp
import time

# Import resources (icon)
from scriptingHelp_rc import *

# For debugging purpose:
STDOUT = sys.stdout

class HelpListItem(QTreeWidgetItem):
    def __init__(self, parent=None):
        QTreeWidgetItem.__init__(self, parent)
        self.details = ""
    def setDetails(self,details_in):
        self.details = details_in
    def getDetails(self):
        return self.details
    

class ScriptingHelpWidget(QWidget):
    def __init__(self, parent=None, lib_path=None):
        QWidget.__init__(self, parent)
        #
        self.bold_font = QFont()
        self.bold_font.setBold(True)
        #
        self.treeView = QTreeWidget(self)
        self.library_menu = QTreeWidgetItem(self.treeView)
        self.library_menu.setText(0,"LIBRARIES")
        self.library_menu.setFont(0,self.bold_font)
        self.library_menu.setTextColor(0, QColor("Blue"));
        self.macro_menu = QTreeWidgetItem(self.treeView)
        self.macro_menu.setText(0,"MACROS")
        self.macro_menu.setFont(0,self.bold_font)
        self.macro_menu.setTextColor(0, QColor("Green"));
        #
        self.textBrowser = QTextBrowser(self)
        self.textBrowser.setMaximumHeight(200)
        #
        self.icon = QLabel(self)
        self.icon.setText(os.getcwd())
        self.icon.setPixmap(QPixmap(":images/DonkiPy.png"))
        self.icon.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.icon.setObjectName("icon")
        #
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout.addWidget(self.treeView, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.textBrowser, 1, 1, 1, 1)
        self.gridLayout.addWidget(self.icon, 0, 0, 1, 1)
        self.setLayout(self.gridLayout)

        '''
        vlayout = QVBoxLayout()
        vlayout.addWidget(self.icon)
        vlayout.addWidget(self.treeView)
        vlayout.addWidget(self.textBrowser)
        self.setLayout(vlayout)
        '''
        #
        self.treeView.setHeaderLabel("DonkiPy Library")
        #
        if (lib_path):
            try:
                idx = sys.path.index(lib_path)
            except:
                sys.path.append(lib_path)
            self.load_all_modules_from_dir(lib_path)
        #
        self.connect(self.treeView,SIGNAL("itemClicked(QTreeWidgetItem*, int)"),self.itemClicked)
        return 
    def load_all_modules_from_dir(self,dirname):
        class_inst = None
	try:
        	mod_files = [x for x in os.listdir(dirname) if x.endswith('.py')]
	except:
		return
        for filename in mod_files:
            filepath = dirname + '/' + filename
            mod_name,file_ext = os.path.splitext(os.path.split(filepath)[-1])
            if file_ext.lower() == '.py':
                py_mod = imp.load_source(mod_name, filepath)
            '''
            elif file_ext.lower() == '.pyc':
                py_mod = imp.load_compiled(mod_name, filepath)
            '''
            if hasattr(py_mod, 'LIBRARY_TYPE'):
                if (py_mod.LIBRARY_TYPE == 'TRANSLATOR'):
                    module_help = QTreeWidgetItem(self.library_menu)
                    module_help.setText(0,mod_name)
                    module_help.setFont(0,self.bold_font)
                    module_help.setTextColor(0, QColor("darkBlue"));
                elif (py_mod.LIBRARY_TYPE == 'MACRO'):
                    module_help = QTreeWidgetItem(self.macro_menu)
                    module_help.setText(0,mod_name)
                    module_help.setFont(0,self.bold_font)
                    module_help.setTextColor(0, QColor("darkGreen"));
                else:
                    module_help = QTreeWidgetItem(self.treeView)
                    module_help.setText(0,mod_name)
                    module_help.setFont(0,self.bold_font)
                    module_help.setTextColor(0, QColor("darkBlue"));
            if hasattr(py_mod, 'ATTRIBUTES_READONLY'):
                self.fill_help_widget(module_help,py_mod.ATTRIBUTES_READONLY,QIcon(":images/attribute_readonly.png"))
            if hasattr(py_mod, 'ATTRIBUTES'):
                self.fill_help_widget(module_help,py_mod.ATTRIBUTES,QIcon(":images/attribute.png"))
            if hasattr(py_mod, 'COMMANDS'):
                self.fill_help_widget(module_help,py_mod.COMMANDS,QIcon(":images/command.png"))
            if hasattr(py_mod, 'MACROS'):
                self.fill_help_widget(module_help,py_mod.MACROS,QIcon(":images/macro.png"))
        return 
    def fill_help_widget(self,treeHeader,dictionary,item_icon):
        for name in dictionary:
             newHelpItem= HelpListItem(treeHeader)
             newHelpItem.setText(0,name)
             newHelpItem.setIcon(0,item_icon);
             tokens = dictionary[name].split("|")
             details = "<b>" + name + "</b><br>"
             if len(tokens) > 1:
                 details += tokens[1]
             else:
                 details += "Description not available"
             details += "<br><br><b>Usage example:</b><br>"
             if len(tokens) > 2:
                 details += tokens[2]
             else:
                 details += "Example not available"
             newHelpItem.setDetails(details)
             self.treeView.addTopLevelItem(newHelpItem)
        return 
    def itemClicked(self,helpItem,col):        
        try:
            self.textBrowser.setText(helpItem.getDetails())
        except:
            self.textBrowser.setText("")
        return 


if __name__ == "__main__":
    from spyderlib.utils.qthelpers import qapplication
    app = qapplication()
    scripting_lib_path = '/home/rob/lavoro/spyder-2.0.12/scripting_library'
    test = ScriptingHelpWidget(lib_path=scripting_lib_path)
    test.show()
    sys.exit(app.exec_())
